/*
  Doc_Arduino - german Arduino Forum
  IDE 1.8.13
  avr-gcc 10.2.0
  17.01.2021
  License: GNU GPLv3
*/

#pragma once

namespace Utilities
{
  namespace Base
  {

    template <class T>
    void formattingBIN (Stream &out, const T &var, const bool separator = false)
    {
      // Anzahl der Bits vom Datentyp T ermitteln und eins abziehen,  //
      // damit wird die Bit Startposition der Maske erzeugt und       //
      // die for Schleife limitiert                                   //
      constexpr uint8_t limit {(sizeof(T) * 8) - 1};
      uint64_t mask {1};   // wegen Overflow Compilerwarnung aufgetrennt
      mask = (mask << limit);
      uint8_t digit {0};

      out.print("0b");
      for (uint8_t i = 0; i <= limit; i++)
      {
        if (var & mask) digit = 1;
        else digit = 0;
        out.print(digit);
        if (separator) 
        {
          // aller 8 Bits ein Punkt dazwischen schieben und das Letzte unterdrücken
          if (!((i + 1) % 8) && (i < limit)) out.print(".");
          // aller 4 Bits ein Hochkomma dazwischen schieben und das Letzte unterdrücken
          else if (!((i + 1) % 4) && (i < limit)) out.print("'");
        }  
        mask = mask >> 1;
      }
    }

    template <class T>
    void formattingHEX (Stream &out, const T &var, const bool separator = false)
    {
      uint64_t data {0};
      bool sig {false};
      const uint8_t len {sizeof(var)};
        
      if (var < 0) {
        sig = true;
        data = static_cast<uint64_t>(var * (-1));
      }
      else {
        data = static_cast<uint64_t>(var);
      }

      uint8_t buf[len+1];
      memset(buf, '\0', len);
      
      // isolate the incoming data type into single bytes
      for (uint8_t i=0; i < len; i++)
      {
        buf[i] = static_cast<uint8_t> (data & 0xFF);
        data = (data >> 8);
      }
        
      if (sig) out.print('-');
      out.print("0x");
     
      for (int8_t i=len-1; i >= 0; i--)
      {
        // ggf. fehlende führende Null vom Datentyp Byte ausgeben
        if (buf[i] <= 0x0F) out.print('0');
        out.print(buf[i], HEX);
        
        if (separator)
        {
          // aller 2 Datentyp Bytes ein Hochkomma dazwischen schieben und das Letzte unterdrücken
          if ((!(i%2)) && (i > 0)) out.print("'");
        }  
      }
    }
  }
}
  
  
template <class T>
void formatBIN (Stream &out, const T &var, const bool separator = false)
{
  Utilities::Base::formattingBIN(out, var, separator);
}

template <class T>
void formatBINln (Stream &out, const T &var, const bool separator = false)
{
  Utilities::Base::formattingBIN(out, var, separator);
  out.println();
}

template <class T>
void formatHEX (Stream &out, const T &var, const bool separator = false)
{
  Utilities::Base::formattingHEX(out, var, separator);
}

template <class T>
void formatHEXln (Stream &out, const T &var, const bool separator = false)
{
  Utilities::Base::formattingHEX(out, var, separator);
  out.println();
}
     
